### packages for the model
library(gamboostLSS)
### package for the graphical representation of the spatial effects
library(BayesX)
### load data
data(india)
### load boundary data for spatial effect
data(india.bnd)
### define vector of quantiles
pp = c(.01, 0.1,0.2, 0.3,0.4,0.5,0.6,0.7,0.8,0.9, .99)
### define list to store boosting objects and vector for storing cross validated risks
qr_boost <- list()
cvr <- vector(length=11)
### start loop for the different quantiles
### Arguments in gamboost():
##### Arguments in formula(): 
####### first argument: dependent variable
####### arguments without further definition: continous variables (BMI of mother and Age of Child)
####### bmrf: baselearner for markov random fields
##### family = QuantReg(tau = pp[i]): quantile regression for current value of pp[i]
##### data: dataset including all used covariates
##### Arguments in control():
####### mstop: number of iterations, determined via 25-fold boostrap (see below)
####### nu: step length, default = 0.1, needs to be to higher values for quantiles
### Arguments in cvrisk:
#### first argument: the model for which the stopping iterations have to be determined
#### grid: grid on which the stopping iterations have to be evaluated
### for more detailed explanations on this algorithm, see boosting tutorial paper:
### Boosting for statistical modelling -- a non-technical introduction
### Mayr, Andreas and Hofner, Benjamin
### Statistical Modelling
### 2017
for(i in 1:11){
  set.seed(1)
  qr_boost_start <-  gamboost(stunting ~ mbmi + cage +
                             bmrf(mcdist, "markov", bnd = india.bnd, center = F),
                             family = QuantReg(tau = pp[i]),
                             data = india,
                             control = boost_control(mstop = 1, nu = 0.5))
  cvrg<- cvrisk(qr_boost_start, grid=1:2000)
  cvr[i] <- mstop(cvrg) 
  qr_boost[[i]] <-gamboost(stunting ~ mbmi + cage +
                             bmrf(mcdist, "markov", bnd = india.bnd, center = F),
                           family = QuantReg(tau = pp[i]),
                           data = india,
                           control = boost_control(mstop = cvr[i], nu = 0.5))
  gc()
}


pdf("resbmi_quant.pdf", height=6, width=9)
### scatterplot in the background (with semitransparent dots)
plot(india$mbmi,
     india$stunting , pch=16, col=rgb(.3,.3,.3,.1),
     ylab="Z-Score", xlab="BMI of the mother", las = 1)
### use order of covariate to plot functions as lines
o_bmi <- order(india$mbmi)
### loop for plotting all 11 functions
for(i in 1:length(pp)){
  ### extract coefficients and design matrix to construct a Z%*%gamma type function
  pl2 <- extract(qr_boost[[i]], "coefficients")[[1]]
  pl <- extract(qr_boost[[i]], "design")[[1]]
  ### calculate prediction for this quantile
  ### centered (minus mean of the predicted function itself)
  ### shifted to the level of the quantile (plus prediction of the whole model)
  pred_cent <- (as.matrix(pl)%*%pl2)[o_bmi] - mean((as.matrix(pl)%*%pl2)[o_bmi]) + mean(qr_boost[[i]]$predict())
  lines(india$mbmi[o_bmi], pred_cent,
        lwd = 2)
}
dev.off()

pdf("resage_quant.pdf", height=6, width=9)
### scatterplot in the background (with semitransparent dots)
plot(india$cage,
     india$stunting ,
     pch=16, col=rgb(.3,.3,.3,.1),
     ylab="Z-Score", xlab="Age", las = 1)
### use order of covariate to plot functions as lines
o_age <- order(india$cage)
### loop for plotting all 11 functions
for(i in 1:length(pp)){
  ### extract coefficients and design matrix to construct a Z%*%gamma type function
  pl2 <- extract(qr_boost[[i]], "coefficients")[[2]]
  pl <- extract(qr_boost[[i]], "design")[[2]]
  ### calculate prediction for this quantile
  ### centered (minus mean of the predicted function itself)
  ### shifted to the level of the quantile (plus prediction of the whole model)
  pred_cent <- (as.matrix(pl)%*%pl2)[o_age] - mean((as.matrix(pl)%*%pl2)[o_age])  + mean(qr_boost[[i]]$predict())
  lines(india$cage[o_age],pred_cent,
        lwd = 2)
}
dev.off()

### Three exemplatory quantiles were chosen to give an impression of the spatial effect
pdf("india_geo.pdf", width=15, height=12)
### bring result in form that is accepted by the function drawmap, i.e. three columns:
#### number of observation
#### name of region of observation as used in both, the spatial variable and the boundary file
#### effect of region of observation
par(mfcol=c(2,3))
### choose pp[3] = 0.2
i <- 3
res <- as.data.frame(cbind(1:length(qr_boost[[i]]$coef()$`bmrf(mcdist, "markov", bnd = india.bnd, center = F)`),
                           as.numeric(names(qr_boost[[i]]$coef()$`bmrf(mcdist, "markov", bnd = india.bnd, center = F)`)), 
                           qr_boost[[i]]$coef()$`bmrf(mcdist, "markov", bnd = india.bnd, center = F)`))
### Arguments in drawmap
##### first argument: data, constructed as explained above
##### map: map used for estimation, boundary file
##### limits: limits for plotting map, when chosen identical for all plots, comparison is easier
##### (choose such that all estimations are within the limits for all models)
drawmap(res, map = india.bnd, col="grey",
        limits=c(-1.2,1.2), main=expression(paste(tau," = ", 0.2)), cex.legend=2.5, cex.main=3)
### box around Bihar, to make clear, where it is
segments(x0 = 84, y0 = 24,
         x1 = 84, y1 = 28, lwd=2, col="black")
segments(x0 = 89, y0 = 24,
         x1 = 89, y1 = 28, lwd=2, col="black")
segments(x0 = 84, y0 = 24,
         x1 = 89, y1 = 24, lwd=2, col="black")
segments(x0 = 84, y0 = 28,
         x1 = 89, y1 = 28, lwd=2, col="black")
#####Zooming into Bihar
drawmap(res, map = india.bnd, col="grey",
        limits = c(-1.2,1.2),
        ylim = c(24,28),
        xlim = c(84, 89))
### choose pp[6] = 0.5
i <- 6
res <- as.data.frame(cbind(1:length(qr_boost[[i]]$coef()$`bmrf(mcdist, "markov", bnd = india.bnd, center = F)`),
                           as.numeric(names(qr_boost[[i]]$coef()$`bmrf(mcdist, "markov", bnd = india.bnd, center = F)`)), 
                           qr_boost[[i]]$coef()$`bmrf(mcdist, "markov", bnd = india.bnd, center = F)`))
drawmap(res, map = india.bnd, col="grey",
        limits=c(-1.2,1.2), main=expression(paste(tau," = ", 0.5)), cex.legend=2.5, cex.main=3)
segments(x0 = 84, y0 = 24,
         x1 = 84, y1 = 28, lwd=2, col="black")
segments(x0 = 89, y0 = 24,
         x1 = 89, y1 = 28, lwd=2, col="black")
segments(x0 = 84, y0 = 24,
         x1 = 89, y1 = 24, lwd=2, col="black")
segments(x0 = 84, y0 = 28,
         x1 = 89, y1 = 28, lwd=2, col="black")
drawmap(res, map = india.bnd, col="grey",
        limits = c(-1.2,1.2),
        ylim = c(24,28),
        xlim = c(84, 89))

### choose pp[9] = 0.8
i <- 9
res <- as.data.frame(cbind(1:length(qr_boost[[i]]$coef()$`bmrf(mcdist, "markov", bnd = india.bnd, center = F)`),
                           as.numeric(names(qr_boost[[i]]$coef()$`bmrf(mcdist, "markov", bnd = india.bnd, center = F)`)), 
                           qr_boost[[i]]$coef()$`bmrf(mcdist, "markov", bnd = india.bnd, center = F)`))
drawmap(res, map = india.bnd, col="grey",
        limits=c(-1.2,1.2), main=expression(paste(tau," = ", 0.8)), cex.legend=2.5, cex.main=3)
segments(x0 = 84, y0 = 24,
         x1 = 84, y1 = 28, lwd=2, col="black")
segments(x0 = 89, y0 = 24,
         x1 = 89, y1 = 28, lwd=2, col="black")
segments(x0 = 84, y0 = 24,
         x1 = 89, y1 = 24, lwd=2, col="black")
segments(x0 = 84, y0 = 28,
         x1 = 89, y1 = 28, lwd=2, col="black")
drawmap(res, map = india.bnd, col="grey",
        limits = c(-1.2,1.2),
        ylim = c(24,28),
        xlim = c(84, 89))

dev.off() 